html, .container {
    height: 100%;
}
body {
    height: 100%;
    background-color: #eee;
}
#control {
    margin:10px;
}
.form-signin {
    max-width: 330px;
    padding: 15px;
    margin: 0 auto;
}
.form-signin .form-signin-heading,
.form-signin .checkbox {
    margin-bottom: 10px;
}
.form-signin .checkbox {
    font-weight: 400;
}
.form-signin .form-control {
    position: relative;
    box-sizing: border-box;
    height: auto;
    padding: 10px;
    font-size: 16px;
}
.form-signin .form-control:focus {
    z-index: 2;
}
.form-signin input[type="email"] {
    margin-bottom: -1px;
    border-bottom-right-radius: 0;
    border-bottom-left-radius: 0;
}
.form-signin input[type="password"] {
    margin-bottom: 10px;
    border-top-left-radius: 0;
    border-top-right-radius: 0;
}
#control {
    margin-top: 10px;
}
.bridge, .auth {
    margin-bottom: 10px;
    background-color: #DDD;
}
.bridge.edited .card {
    background-color: #ffd6cc;
}
table.targets {
    margin: 5px 0 5px 0;
}
tr.target {
    background-color: #EEE !important;
}
tr.edited {
    background-color: #ffd6cc !important;
}
.auth.edited .card {
    background-color: #ffd6cc;
}
#add-btn {
    margin-bottom:10px;
}
.source {
    padding-left: 10px;
}
.source h4, .auth h4 {
    display:inline;
}
.targets {
    margin-top: 10px;
}
table.targets {
    border-collapse: separate !important;
}
.target-prop{}
td.target-actions {
    text-align: right;
}
.modal-body td {
    text-align: right;
    vertical-align: middle;
}
#footer-bar {
    background-color: #DBDBDB;
}
.flash-msg {
    z-index: 1060 !important;
    margin-top:30px;
    margin-left:auto;
    margin-right:auto;
    width:65%;
    padding: 30px;
}
textarea.form-control {}

/* loader */
#loader-overlay {
    position: fixed;
    display: block;
    width: 100%;
    height: 100%;
    top: 0;
    left: 0;
    right: 0;
    bottom: 0;
    background-color: rgba(0,0,0,0.5);
    z-index: 1000 !important;
    cursor: pointer;
}
.loader-box {
    position: absolute;
    width: 50px;
    height: 50px;
    z-index: 1060 !important;
    top:80px;
    left: 0;
    right: 0;
    margin-left:auto;
    margin-right:auto;
}
.loader {
    position: relative;
    border: 16px solid #f3f3f3;
    border-radius: 50%;
    border-top: 16px solid #3498db;
    width: 100%;
    height: 100%;
    -webkit-animation: spin 1s linear infinite; /* Safari */
    animation: spin 1s linear infinite;
}
/* Safari */
@-webkit-keyframes spin {
  0% { -webkit-transform: rotate(0deg); }
  100% { -webkit-transform: rotate(360deg); }
}
@keyframes spin {
  0% { transform: rotate(0deg); }
  100% { transform: rotate(360deg); }
}

/* vue */
.fade-enter-active, .fade-leave-active {
    transition: opacity .5s;
}
.fade-enter, .fade-leave-to /* .fade-leave-active below version 2.1.8 */ {
    opacity: 0;
}
